# -*- coding: utf-8 -*-
"""
Tests for filters in enterprise_data.
"""


import mock
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITestCase

from enterprise_data.models import EnterpriseEnrollment
from enterprise_data.tests.mixins import JWTTestMixin
from enterprise_data.tests.test_utils import UserFactory


class TestConsentGrantedFilterBackend(JWTTestMixin, APITestCase):
    """
    Test suite for the ``TestConsentGrantedFilterBackend`` filter.
    """
    fixtures = ('enterprise_enrollment', 'enterprise_user', )

    def setUp(self):
        super(TestConsentGrantedFilterBackend, self).setUp()
        self.user = UserFactory(is_staff=True)
        self.client.force_authenticate(user=self.user)  # pylint: disable=no-member
        self.enterprise_id = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.url = reverse('v0:enterprise-enrollments-list',
                           kwargs={'enterprise_id': self.enterprise_id})
        self.set_jwt_cookie(context=self.enterprise_id)

    def mock_enterprise_api_client_response(self, enforce_dsc=None):
        """
        Mocks enterprise api client response.
        """
        mock_path = 'enterprise_data.filters.EnterpriseApiClient.get_enterprise_customer'
        with mock.patch(mock_path) as mock_enterprise_api_client:
            mock_enterprise_api_client.return_value = {
                'uuid': self.enterprise_id,
                'enable_audit_data_reporting': True,
                'enforce_data_sharing_consent': enforce_dsc or True,
            }
            return self.client.get(self.url)

    def test_filter_for_list(self):
        """
        Filter users through email/username if requesting user is staff, otherwise based off of request user ID.
        """
        response = self.mock_enterprise_api_client_response()

        assert response.status_code == status.HTTP_200_OK
        data = response.json()

        # Fixture data for enterprise 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c' contains 3 objects but only 2 with consent
        assert EnterpriseEnrollment.objects.filter(enterprise_id=self.enterprise_id).count() == 3
        assert len(data['results']) == 2

    def test_filter_with_external_consent(self):
        """
        If the enterprise is configured for externally managed data consent, all enrollments will be returned.
        """
        response = self.mock_enterprise_api_client_response('externally_managed')
        assert response.status_code == status.HTTP_200_OK
        data = response.json()

        # Fixture data for enterprise 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c' contains 3 objects but only 2 with consent
        assert EnterpriseEnrollment.objects.filter(enterprise_id=self.enterprise_id).count() == 3
        assert len(data['results']) == 3



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-06-12 03:36


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0005_auto_20180524_2204'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='last_activity_timestamp',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='user_country_code',
            field=models.CharField(max_length=2, null=True),
        ),
    ]



"""
Enterprise_data url configuration.
"""


from django.conf.urls import include, url
from django.contrib import admin

app_name = 'enterprise_data'
urlpatterns = [
    url(
        r'^enterprise/api/',
        include('enterprise_data.api.urls'),
        name='enterprise_data_api'
    ),
    url(r"^admin/", admin.site.urls),
]



# -*- coding: utf-8 -*-
"""
Serializers for enterprise api version 0 endpoint.
"""


from datetime import date

from rest_framework import serializers

from enterprise_data.models import EnterpriseEnrollment, EnterpriseUser


class EnterpriseEnrollmentSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseEnrollment model.
    """
    course_api_url = serializers.SerializerMethodField()
    unenrollment_end_within_date = serializers.SerializerMethodField()
    progress_status = serializers.SerializerMethodField()
    has_passed = serializers.BooleanField(default=False, write_only=True)

    def get_course_api_url(self, obj):
        """Constructs course api url"""
        return '/enterprise/v1/enterprise-catalogs/{enterprise_id}/courses/{course_id}'.format(
            enterprise_id=obj.enterprise_id, course_id=obj.course_id
        )

    def get_unenrollment_end_within_date(self, obj):
        """
        Return "True" when un-enrolled date is within 14 days of course start or enrollment date
        (whichever is later), else return "False" and return "None" when there is no un-enrollment timestamp.
        """
        unenrollment_within_date = None
        if obj.unenrollment_timestamp:
            unenrollment_within_date = (
                not obj.course_start or
                (obj.unenrollment_timestamp - obj.course_start).days <= 14 or
                (obj.unenrollment_timestamp - obj.enrollment_created_timestamp).days <= 14
            )

        return unenrollment_within_date

    def get_progress_status(self, obj):
        """
        Return "Passed" if "has_passed" is True.
        Return "In Progress" if "has_passed" is False and course end date has not passed.
        Return "Failed" if "has_passed" is False and course end date has already passed.
        """
        progress_status = "In Progress"
        if obj.has_passed:
            progress_status = "Passed"
        elif obj.course_end and obj.course_end.date() < date.today():
            progress_status = "Failed"
        return progress_status

    class Meta:
        model = EnterpriseEnrollment
        exclude = ('created', )


class EnterpriseUserSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseUser model.
    """

    class Meta:
        model = EnterpriseUser
        exclude = ('created', )

    def to_representation(self, instance):
        representation = super(EnterpriseUserSerializer, self).to_representation(instance)

        if hasattr(instance, 'enrollment_count'):
            representation['enrollment_count'] = instance.enrollment_count
        if hasattr(instance, 'course_completion_count'):
            representation['course_completion_count'] = instance.course_completion_count

        return representation


class LearnerCompletedCoursesSerializer(serializers.Serializer):    # pylint: disable=abstract-method
    """
    Serializer for learner's completed courses.
    """
    user_email = serializers.EmailField()
    completed_courses = serializers.IntegerField()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-04 16:21


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0013_auto_20180831_1931'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterpriseuser',
            name='created',
            field=models.DateTimeField(null=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Enterprise Data Django application initialization.
"""

from django.apps import AppConfig


class EnterpriseDataAppConfig(AppConfig):
    """
    Configuration for the enterprise Django application.
    """

    name = "enterprise_data"



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-04-30 13:58


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='enterpriseenrollment',
            old_name='enterprise_sso_user_id',
            new_name='enterprise_sso_uid',
        ),
        migrations.AddField(
            model_name='enterpriseenrollment',
            name='enterprise_sso_site_id',
            field=models.PositiveIntegerField(null=True),
        ),
    ]



"""
management command for creating enterprise users
"""


import sys

from django.core.management.base import BaseCommand, CommandError

import enterprise_data.tests.test_utils


class Command(BaseCommand):
    """ management command class """
    help = 'Creates an EnterpriseUser with randomized attributes'

    def add_arguments(self, parser):
        parser.add_argument(
            'enterprise_id',
            type=str,
            help='UUID for an enterprise'
        )

    def handle(self, *args, **options):
        enterprise_id = options['enterprise_id']
        try:
            ent_user = enterprise_data.tests.test_utils.EnterpriseUserFactory(
                enterprise_id=enterprise_id
            )
        except Exception as exc:
            info = (
                'Error trying to create EnterpriseUser with uuid '
                '{}: {}'.format(enterprise_id, exc)
            )
            raise CommandError(info)

        info = '\n\nCreated EnterpriseUser with id {} for enterprise with uuid {}\n'.format(
            ent_user.enterprise_user_id,
            ent_user.enterprise_id
        )
        sys.stdout.write(info)

        info = (
            'You can create some enrollments for this user by running the following '
            'command:\n\n    ./manage.py create_enterprise_enrollment {} {}\n\n'.format(
                ent_user.enterprise_id,
                ent_user.enterprise_user_id,
            )
        )
        sys.stdout.write(info)



"""
Filters for enterprise data views.
"""


from rest_framework import filters

from django.db.models import Q

from enterprise_data.clients import EnterpriseApiClient

# Admittedly this is sort of hacky because the use of "|" with 2 Q objects
# forces the ORM to use a LEFT OUTER JOIN, which is needed to return a user
# that has multiple enrollments where at least one has consent_granted=True
CONSENT_TRUE_OR_NOENROLL_Q = Q(enrollments__consent_granted=True) | Q(enrollments__isnull=True)


class FiltersMixin:
    """
    Util mixin for enterprise_data filters.
    """

    def update_session_with_enterprise_data(self, request):
        """
        Make cached call to lms to get an enterprise data and update request session.
        """
        enterprise_client = EnterpriseApiClient(request.auth)
        __ = enterprise_client.get_enterprise_and_update_session(request)


class ConsentGrantedFilterBackend(filters.BaseFilterBackend, FiltersMixin):
    """
    Filter backend for any view that needs to filter results where consent has not been granted.

    This requires that `CONSENT_GRANTED_FILTER` be set in the view as a class variable, to identify
    the object's relationship to the consent_granted field.

    If the enterprise is configured for externally managed data sharing consent, all enrollments will be
    returned
    """

    def filter_queryset(self, request, queryset, view):
        """
        Filter a queryset for results where consent has been granted.
        """
        current_enterprise = view.kwargs.get('enterprise_id', None)

        self.update_session_with_enterprise_data(request)

        data_sharing_enforce = request.session.get('enforce_data_sharing_consent', {})
        # if the enterprise is configured for "externally managed" data sharing consent,
        # ignore the consent_granted column.
        if data_sharing_enforce.get(current_enterprise, '') != 'externally_managed':
            filter_kwargs = {view.CONSENT_GRANTED_FILTER: True}
            queryset = queryset.filter(**filter_kwargs)
        return queryset


class AuditEnrollmentsFilterBackend(filters.BaseFilterBackend, FiltersMixin):
    """
    Filter backend to exclude enrollments where enrollment mode is `audit`.

    This requires that `ENROLLMENT_MODE_FILTER` be set in the view as a class
    variable, to identify the object's relationship to the
    `user_current_enrollment_mode` field.
    """

    def filter_queryset(self, request, queryset, view):
        """
        Filter out queryset for results where enrollment mode is `audit`.
        """
        enterprise_id = view.kwargs['enterprise_id']

        self.update_session_with_enterprise_data(request)

        enable_audit_data_reporting = request.session['enable_audit_data_reporting'].get(enterprise_id, False)

        if not enable_audit_data_reporting:
            # Filter out enrollments that have audit mode and do not have a coupon code or an offer.
            filter_query = {
                view.ENROLLMENT_MODE_FILTER: 'audit',
                view.COUPON_CODE_FILTER: None,
                view.OFFER_FILTER: None
            }
            queryset = queryset.exclude(**filter_query)

        return queryset


class AuditUsersEnrollmentFilterBackend(filters.BaseFilterBackend, FiltersMixin):
    """
    Filter backend to filter Enterprise Users with 'audit' mode enrollments.
    """

    def filter_queryset(self, request, queryset, view):
        """
        Filter out queryset on the basis of its enrollment mode.

        If `enable_audit_data_reporting` is not enabled then it will exclude the Users with 'audit' mode enrollment.
        """
        enterprise_id = view.kwargs['enterprise_id']

        self.update_session_with_enterprise_data(request)

        enable_audit_data_reporting = request.session['enable_audit_data_reporting'].get(enterprise_id, False)

        return queryset if enable_audit_data_reporting else queryset.filter(
            ~Q(enrollments__user_current_enrollment_mode='audit')
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-05-01 09:28


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0003_auto_20180501_0603'),
    ]

    operations = [
        migrations.RenameField(
            model_name='enterpriseenrollment',
            old_name='user_account_creation_date',
            new_name='user_account_creation_timestamp',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2019-03-29 15:53


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data_roles', '0003_add_role_based_access_control_switch'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisedataroleassignment',
            name='enterprise_id',
            field=models.UUIDField(blank=True, null=True, verbose_name='Enterprise Customer UUID'),
        ),
    ]



"""
Tests for create_enterprise_user management command
"""


from unittest import TestCase

from mock import mock
from pytest import mark

from django.core.management import call_command

from enterprise_data.models import EnterpriseUser
from enterprise_data.tests.test_utils import EnterpriseUserFactory


@mark.django_db
class TestCreateEnterpriseUserCommand(TestCase):
    """ test class here """

    def setUp(self):
        super(TestCreateEnterpriseUserCommand, self).setUp()
        self.uuid = 'a'*32

    def test_create_enterpriseuser(self):
        """
        Manangement command should successfully be able to create EnterpriseUser
        """
        assert EnterpriseUser.objects.count() == 0

        args = [self.uuid]
        call_command('create_enterprise_user', *args)

        assert EnterpriseUser.objects.count() == 1
        assert EnterpriseUser.objects.filter(enterprise_id=args[0]).count() == 1

    def test_create_enterpriseuser_error(self):
        """
        Manangement command should successfully be able to create EnterpriseUser
        """
        EnterpriseUserFactory(enterprise_id=self.uuid)
        assert EnterpriseUser.objects.count() == 1

        args = [self.uuid]
        with mock.patch('enterprise_data.tests.test_utils.EnterpriseUserFactory') as mock_factory:
            mock_factory.side_effect = [Exception]
            with self.assertRaises(Exception):
                call_command('create_enterprise_user', *args)

        assert EnterpriseUser.objects.count() == 1



# -*- coding: utf-8 -*-
"""
Client for connecting to the LMS Enterprise endpoints.
"""



import os
from collections import OrderedDict

from enterprise_reporting.clients import EdxOAuth2APIClient


class EnterpriseAPIClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Enterprise API.
    """

    API_BASE_URL = EdxOAuth2APIClient.LMS_ROOT_URL + '/enterprise/api/v1/'
    APPEND_SLASH = True

    ENTERPRISE_REPORTING_ENDPOINT = 'enterprise_customer_reporting'
    ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT = 'enterprise_catalogs'

    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    @EdxOAuth2APIClient.refresh_token
    def get_all_enterprise_reporting_configs(self, **kwargs):
        """
        Query the enterprise customer reporting endpoint for all available configs.
        """
        return self._load_data(
            self.ENTERPRISE_REPORTING_ENDPOINT,
            should_traverse_pagination=True,
            **kwargs
        )

    @EdxOAuth2APIClient.refresh_token
    def get_enterprise_reporting_configs(self, enterprise_customer_uuid, **kwargs):
        """
        Query the enterprise customer reporting endpoint for a given enterprise customer.
        """
        return self._load_data(
            self.ENTERPRISE_REPORTING_ENDPOINT,
            querystring={'enterprise_customer': enterprise_customer_uuid},
            should_traverse_pagination=True,
            **kwargs
        )

    @EdxOAuth2APIClient.refresh_token
    def get_content_metadata(self, enterprise_customer_uuid, reporting_config):
        """Return all content metadata contained in the catalogs associated with an Enterprise Customer."""
        content_metadata = OrderedDict()

        enterprise_customer_catalogs = extract_catalog_uuids_from_reporting_config(reporting_config)
        if not enterprise_customer_catalogs.get('results'):
            enterprise_customer_catalogs = self._load_data(
                self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
                should_traverse_pagination=True,
                querystring={
                    'enterprise_customer': enterprise_customer_uuid,
                    'page_size': self.PAGE_SIZE,
                },
            )
        for catalog in enterprise_customer_catalogs.get('results', []):
            catalog_content = self._load_data(
                self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
                resource_id=catalog['uuid'],
                should_traverse_pagination=True,
                querystring={'page_size': self.PAGE_SIZE},
            )
            # It's possible that there are duplicate items.
            # Filter them out by assigning common items to their common identifier in a dictionary.
            for item in catalog_content['results']:
                key = 'uuid' if item['content_type'] == 'program' else 'key'
                content_metadata[item[key]] = item

        # We only made this a dictionary to help filter out duplicates by a common key. We just want values now.
        return list(content_metadata.values())


def extract_catalog_uuids_from_reporting_config(reporting_config):
    """
    Helper method to extract uuids from reporting config

    Returns a dict with 1 key, 'results', whose value is a list of
    dicts containing a key-value pair of 'uuid' and some uuid
    """
    enterprise_customer_catalogs = {'results': [
        {'uuid': catalog['uuid']}
        for catalog in reporting_config.get('enterprise_customer_catalogs', [])
        ]
    }
    return enterprise_customer_catalogs


class EnterpriseDataApiClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Enterprise Data API.
    """

    API_BASE_URL = os.getenv('ANALYTICS_API_URL', default='') + '/enterprise/api/v0'
    APPEND_SLASH = True

    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    @EdxOAuth2APIClient.refresh_token
    def get_enterprise_enrollments(self, enterprise_customer_uuid):
        return self._load_data(
            'enterprise',
            resource_id=enterprise_customer_uuid,
            detail_resource='enrollments',
            should_traverse_pagination=True,
            querystring={'page_size': self.PAGE_SIZE},
        )


class AnalyticsDataApiClient(EdxOAuth2APIClient):
    """
    Client for connecting to the Analytics Data API.
    """

    API_BASE_URL = os.getenv('ANALYTICS_API_URL', default='') + '/api/v0'
    APPEND_SLASH = True

    PAGE_SIZE = os.getenv('PAGE_SIZE', default=1000)

    @EdxOAuth2APIClient.refresh_token
    def get_enterprise_engagements(self, enterprise_customer_uuid):
        return self._load_data(
            'enterprise',
            resource_id=enterprise_customer_uuid,
            detail_resource='engagements',
            should_traverse_pagination=True,
            querystring={'page_size': self.PAGE_SIZE},
        )



# -*- coding: utf-8 -*-
"""
Tests for the `edx-enterprise` serializer module.
"""

from datetime import timedelta

import ddt
from pytest import mark, raises
from rest_framework.test import APITransactionTestCase

from django.utils import timezone

from enterprise_data.api.v0.serializers import EnterpriseEnrollment, EnterpriseEnrollmentSerializer
from enterprise_data.tests.test_utils import EnterpriseUserFactory


@mark.django_db
@ddt.ddt
class TestEnterpriseEnrollmentSerializer(APITransactionTestCase):
    """
    Tests for `enterprise_enrollment` API serializer.
    """

    def setUp(self):
        super(TestEnterpriseEnrollmentSerializer, self).setUp()

        EnterpriseUserFactory(enterprise_user_id=1)
        self.enrollment_data = {
            "id": 1,
            "enterprise_id": "ee5e6b3a-069a-4947-bb8d-d2dbc323396c",
            "enterprise_name": "Enterprise 1",
            "lms_user_id": 11,
            "enterprise_user": 1,
            "course_id": "edX/Open_DemoX/edx_demo_course",
            "enrollment_created_timestamp": "2014-06-27T16:02:38Z",
            "unenrollment_timestamp": None,
            "user_current_enrollment_mode": "verified",
            "consent_granted": 1,
            "letter_grade": "Pass",
            "progress_status": "Failed",
            "passed_timestamp": "2017-05-09T16:27:34.690065Z",
            "enterprise_sso_uid": "harry",
            "course_title": "All about acceptance testing!",
            "course_start": "2016-09-01T00:00:00Z",
            "course_end": "2016-12-01T00:00:00Z",
            "course_pacing_type": "instructor_paced",
            "course_duration_weeks": "8",
            "course_min_effort": 2,
            "course_max_effort": 4,
            "user_account_creation_timestamp": "2015-02-12T23:14:35Z",
            "user_email": "test@example.com",
            "user_username": "test_user",
            "course_key": "edX/Open_DemoX",
            "enterprise_site_id": 1,
            "current_grade": 0.80,
            "discount_price": "120.00",
            "last_activity_date": "2017-06-23",
            "coupon_code": "PIPNJSUK33P7PTZH",
            "user_country_code": "US",
            "course_price": "200.00",
            "coupon_name": "Enterprise Entitlement Coupon",
            "offer": "Percentage, 100 (#1234)",
            "unenrollment_end_within_date": None,
        }

        self.course_api_url = '/enterprise/v1/enterprise-catalogs/{enterprise_id}/courses/{course_id}'.format(
            enterprise_id=self.enrollment_data['enterprise_id'], course_id=self.enrollment_data['course_id']
        )

    def test_enrollment_serialization(self):
        expected_serialized_data = dict(self.enrollment_data, course_api_url=self.course_api_url)

        serializer = EnterpriseEnrollmentSerializer(data=self.enrollment_data)
        serializer.is_valid()
        serializer.save()

        enterprise_enrollment_id = EnterpriseEnrollment.objects.first().id
        expected_serialized_data['id'] = enterprise_enrollment_id
        assert serializer.data == expected_serialized_data

    @ddt.data(
        # No course start date
        (None, '2016-12-01T00:00:00Z', '2016-12-01T00:00:00Z', True),
        # Same day enroll/un-enroll
        ('2016-12-14T00:00:00Z', '2016-12-01T00:00:00Z', '2016-12-01T01:00:00Z', True),
        # Un-enroll on next day of enrollment
        ('2016-12-14T00:00:00Z', '2016-12-01T00:00:00Z', '2016-12-02T01:00:00Z', True),
        # Un-enroll on same day as course start
        ('2016-12-14T00:00:00Z', '2016-12-01T00:00:00Z', '2016-12-14T01:00:00Z', True),
        # Un-enroll on same day as course start and enrollment
        ('2016-12-14T00:00:00Z', '2016-12-14T00:00:00Z', '2016-12-14T01:00:00Z', True),
        # Un-enroll on 14th day of course start
        ('2016-12-14T00:00:00Z', '2016-12-01T00:00:00Z', '2016-12-28T01:00:00Z', True),
        # Un-enroll on 15th day of course start
        ('2016-12-14T00:00:00Z', '2016-12-01T00:00:00Z', '2016-12-29T00:00:00Z', False),
        # Un-enroll on 14th day of enrollment
        ('2016-12-14T00:00:00Z', '2016-12-15T00:00:00Z', '2016-12-29T00:00:00Z', True),
        # Un-enroll on 15th day of enrollment
        ('2016-12-14T00:00:00Z', '2016-12-15T00:00:00Z', '2016-12-30T00:00:00Z', False),
        # Un-enroll earlier than enrollment
        ('2016-12-14T00:00:00Z', '2016-12-15T00:00:00Z', '2016-12-14T00:00:00Z', True),
    )
    @ddt.unpack
    def test_unenrollment_end_within_date(
            self, course_start, enrollment_created_timestamp, unenrollment_timestamp,
            expected_unenrollment_end_within_date
    ):
        self.enrollment_data['course_start'] = course_start
        self.enrollment_data['unenrollment_timestamp'] = unenrollment_timestamp
        self.enrollment_data['enrollment_created_timestamp'] = enrollment_created_timestamp

        serializer = EnterpriseEnrollmentSerializer(data=self.enrollment_data)
        serializer.is_valid()
        serializer.save()
        self.assertEqual(expected_unenrollment_end_within_date, serializer.data['unenrollment_end_within_date'])

    @ddt.data(
        # No course end date and has_passed is False
        (None, False, 'In Progress'),
        # No course end date and has_passed is True
        (None, True, 'Passed'),
        # Course not ended and has_passed is False
        (timezone.now() + timedelta(days=1), False, 'In Progress'),
        # Course about to end today and has_passed is False
        (timezone.now(), False, 'In Progress'),
        # Course already ended and has_passed is False
        (timezone.now() + timedelta(days=-1), False, 'Failed'),
        # Course already ended and has_passed is True
        (timezone.now() + timedelta(days=-1), True, 'Passed'),
        # Course not ended and has_passed is True
        (timezone.now() + timedelta(days=1), True, 'Passed'),
    )
    @ddt.unpack
    def test_progress_status(self, course_end, has_passed, expected_progress_status):
        self.enrollment_data['course_end'] = course_end
        self.enrollment_data['has_passed'] = has_passed

        serializer = EnterpriseEnrollmentSerializer(data=self.enrollment_data)
        serializer.is_valid()
        serializer.save()
        self.assertEqual(expected_progress_status, serializer.data['progress_status'])



# -*- coding: utf-8 -*-
"""
Django admin integration for enterprise-data.
"""

from edx_rbac.admin import UserRoleAssignmentAdmin, UserRoleAssignmentAdminForm

from django.contrib import admin

from enterprise_data_roles.models import EnterpriseDataRoleAssignment


class EnterpriseDataRoleAssignmentAdminForm(UserRoleAssignmentAdminForm):
    """
    Django admin form for EnterpriseDataRoleAssignmentAdmin.
    """

    class Meta:
        """
        Meta class for EnterpriseDataRoleAssignmentAdminForm.
        """

        model = EnterpriseDataRoleAssignment
        fields = "__all__"


@admin.register(EnterpriseDataRoleAssignment)
class EnterpriseDataRoleAssignmentAdmin(UserRoleAssignmentAdmin):
    """
    Django admin for EnterpriseDataRoleAssignment Model.
    """

    class Meta:
        """
        Meta class for EnterpriseDataRoleAssignmentAdmin.
        """

        model = EnterpriseDataRoleAssignment

    fields = ('user', 'role', 'enterprise_id')
    form = EnterpriseDataRoleAssignmentAdminForm
